module MeterpreterDetection_Certificates;

export {
	global ou_meterpreter_regex = /OU=(cross-platform|synthesizing|transmitting|open-source|solid-state|application|transmitter|compressing|quantifying|calculating|programming|multi-byte|hard-drive|synthesize|backing-up|overriding|navigating|connecting|generating|auxiliary|redundant|bluetooth|bandwidth|microchip|interface|capacitor|calculate|bypassing|back-end|wireless|protocol|firewall|override|compress|navigate|generate|quantify|transmit|indexing|primary|digital|virtual|optical|program|monitor|circuit|back-up|connect|program|hacking|copying|parsing|online|haptic|neural|mobile|driver|system|sensor|matrix|bypass|reboot|1080p|panel|array|pixel|alarm|index|input|parse|port|card|feed|hack|copy|bus)(,|\/)/;

	global email_meterpreter_regex = /emailAddress=(cross-platform|synthesizing|transmitting|open-source|solid-state|application|transmitter|compressing|quantifying|calculating|programming|multi-byte|hard-drive|synthesize|backing-up|overriding|navigating|connecting|generating|auxiliary|redundant|bluetooth|bandwidth|microchip|interface|capacitor|calculate|bypassing|back-end|wireless|protocol|firewall|override|compress|navigate|generate|quantify|transmit|indexing|primary|digital|virtual|optical|program|monitor|circuit|back-up|connect|program|hacking|copying|parsing|online|haptic|neural|mobile|driver|system|sensor|matrix|bypass|reboot|1080p|panel|array|pixel|alarm|index|input|parse|port|card|feed|hack|copy|bus)@/;
}

# Whether the OU field matches a Meterpreter-generated keyword.
function bad_words(subject: string, issuer: string): bool
	{
	if ( ou_meterpreter_regex in subject && ou_meterpreter_regex in issuer )
		if ( email_meterpreter_regex in subject &&
		     email_meterpreter_regex in issuer )
			return T;

	return F;
	}

function x509_certificate_helper(conns: table[conn_id] of connection,
    cert: X509::Certificate)
	{
	if ( bad_words(cert$subject, cert$issuer) )
		{
		for ( c in conns )
			{
			local message = fmt("Meterpreter detected with reason=CERTIFICATE_DETECTED, protocol=HTTPS, os=ANY");
			MeterpreterDetection::log_a_notice(conns[c], message,
			    MeterpreterDetection::Meterpreter_Detected, T);
			}
		}
	}

event x509_certificate(f: fa_file, cert_ref: opaque of x509,
    cert: X509::Certificate)
	{
	x509_certificate_helper(f$conns, cert);
	}
